import { ChangeDetectorRef, Component } from '@angular/core';
import 'cordova-plugin-purchase/www/store.d';
import catalog from "./iapCatalog.config";
import { DialogService } from '../service/helpers/dialogs.service';
import { Logger } from '../classes/Logger/logger';

type IAPProduct = CdvPurchase.Product;

@Component({
  selector: 'app-iap',
  templateUrl: './iap.component.html',
  styleUrls: ['./iap.component.css']
})

export class IAPComponent {
  enableIAPMenu: boolean;
  enableIAPProduct:boolean;
  shownProduct:IAPProduct;
  logger:Logger = new Logger();

  get products(): IAPProduct[] {
      return CdvPurchase.store.products;
  }

  get enableBlur() : boolean {
    return this.enableIAPMenu || this.enableIAPProduct;
  }

  constructor(private ref: ChangeDetectorRef,
    private helper: DialogService,
    ) {
      this.logger.context("-IAP-");
      this.checkReady().then(() => {
        if(!window.CdvPurchase) {
          this.logger.error("CdvPurchase is NOT defined???");
          return;
        }
        (CdvPurchase as any).createdAt = new Date();
        (window as any)._CdvPurchase = CdvPurchase;
        this.logger.log(window.CdvPurchase === CdvPurchase ? 'same object' : 'different objects');
        this.logger.log(CdvPurchase);
        this.logger.log("CdvPurchase IS defined!!");
        const store = CdvPurchase.store;
        store.verbosity = CdvPurchase.LogLevel.DEBUG;

        const iaptic = new CdvPurchase.Iaptic({
          appName: "app.lostseraph.linker",
          apiKey: "4bd3afae-4204-4767-a058-9c723f667171",
        });

        store.validator = iaptic.validator;

        this.registerProducts();
        this.setupListeners();

        this.logger.log("Store initializing...")
        store.initialize([CdvPurchase.Platform.TEST,
          CdvPurchase.Platform.APPLE_APPSTORE
        ])
        .then(() => {

          const appStore = store.getAdapter(CdvPurchase.Platform.APPLE_APPSTORE);

          // if(!appStore) {
          //   this.logger.warn("AppStore not defined?");
          // } else if(!appStore.checkSupport('order')) {
          //   this.logger.warn("AppStore users CAN'T make payments???")
          // } else {
          //   this.logger.log("AppStore users can make payments");
          // }

          this.appStoreRedemptionSheet();

          this.logger.log("Store initialize done...");
          this.ref.detectChanges();
        });

        CdvPurchase.store.error((error) => {
          this.logger.error('CdvPurchase ERROR: ' + error);
        });
      });
  }

  registerProducts() {
    CdvPurchase.store.register(catalog.product);
  }

  debug_Purchases() {
    (CdvPurchase as any).updatedAt = new Date();
    let debugLog = "";
    debugLog += "\nTESTS:\n____________________________";
    debugLog += "\nlist: " + (!!CdvPurchase.store?.products?.length);
    debugLog += "\nactual product: " + CdvPurchase.store.get("app.sgl.linker.iap.test.1", CdvPurchase.Platform.APPLE_APPSTORE);
    debugLog += "\ntest product: " + CdvPurchase.store.get("test-non-consumable", CdvPurchase.Platform.TEST);
    this.logger.log(debugLog);
  }

  appStoreRedemptionSheet() {
    const appStore = CdvPurchase.store.getAdapter(CdvPurchase.Platform.APPLE_APPSTORE) as CdvPurchase.AppleAppStore.Adapter;
    if (appStore?.ready) {
        appStore.presentCodeRedemptionSheet();
        return;
    }
    this.logger.log("appStore adapter is not ready at this point...");
}
  setupListeners() {
    // General query to all products
    const storeEvents = CdvPurchase.store.when();

    if(!!storeEvents.approved) {
      storeEvents.approved(t => {
        // Handle the product deliverable
        t.products.forEach(this.productApproved, this);
        this.ref.detectChanges();
        return t.verify();
      });
    }

    if(!!storeEvents.verified) {
      storeEvents.verified((t: CdvPurchase.VerifiedReceipt) => {
        this.logger.log(`verified: ${t.id}`);
        t.finish();
      });
    }

    if(!!storeEvents.finished) {
      storeEvents.finished( (t: CdvPurchase.Transaction) => {
        t.products.forEach(this.updateProfile, this);
      });
    }

    if(!!storeEvents.receiptUpdated) {
      storeEvents.receiptUpdated(r => this.updatePurchases(r));
    }

    if(!!storeEvents.productUpdated) {
      storeEvents.productUpdated(p => this.ref.detectChanges());
    }

    // Specific query for one ID
    // CdvPurchase.store.when(PRODUCT_PRO_KEY).owned((p: IAPProduct) => {
    //   // run this for each product in catalog
    // });
  }

  updateUI(product : CdvPurchase.Product) {
    const pricing = product.pricing;
    let priceString = "no price";

    if(pricing) {
      priceString = `price: ${pricing.price} ${pricing.currency}`;
    }

    this.logger.log(`- title ${product.title}\n\t${priceString}`);
  }

  updatePurchases(receipt: CdvPurchase.Receipt) {
    this.logger.log("running update purchases...");
    receipt.transactions.forEach(transaction => {
      transaction.products.forEach(trProduct => {
        this.logger.log(`product owned: ${trProduct.id}`);
      });
    });
  }

  purchase(product: IAPProduct | string) {
    let offer:CdvPurchase.Offer | undefined;
    let id = "";

    if(typeof(product) === "string" ) {
      id = product;
    }
    else {
      id = product.id;
    }
    const prod = CdvPurchase.store.get(id);

    if(!prod) {
      return;
    }

    if(prod.getOffer){
      offer = prod.getOffer();
    }
    else if(prod.offers && prod.offers.length) {
      offer = prod.offers[0];
    }

    if(offer){
      CdvPurchase.store.order(offer).then(() => {
        // Purchase in progress!
      }, (e: any) => {
        this.logger.log(e);
        this.helper.presentAlert('Failed', `Failed to purchase: ${e}`);
      });
    }
  }

  // To comply with AppStore rules
  restorePurchases() {
    CdvPurchase.store.restorePurchases();
  }

  showMenu() {
    this.enableIAPMenu = true;
  }

  hideMenu() {
    this.enableIAPMenu = false;
  }

  showProduct(product: IAPProduct) {
    this.enableIAPProduct = true;
    this.shownProduct = product;
  }

  hideProduct() {
    this.enableIAPProduct = false;
  }

  async checkReady() {
    return new Promise<void>(resolve => {
      const wait = () => {
        if(!window.CdvPurchase || !window.CdvPurchase.store) {
          setTimeout(() => {
            wait();
          }, 100);
        }
        else {
          resolve();
        }
      }

      wait();
    });
  }

  productApproved(p:{ id:string, offerId?:string }) {
    // send to user profile.
    this.helper.presentAlert('Approved', `product ${p.id} has been approved.`);
  }

  updateProfile(p:{ id:string, offerId?:string }) {
    // send to user profile.
    this.helper.presentAlert('Success', `sending product ${p.id} off to user profile now`);
  }
}
